% GenTSP test program by m.takigahira

clear;

% wav file settings
Fs = 16000;
bit = 24;

% TSP signal length
N = Fs * 2;
m = N / 4;
f = (N - (Fs * 1.024)) / 2;
g = f + (Fs * 1.024) - 1;
f = round(f); % for case of not multiples of 16kHz freq. e.g.) 44.1kHz
g = round(g);

% Make phase
k1 = 0 : N / 2;
omega1 = 4 * pi * m * (k1 / N) .^ 2;
k2 = N / 2 + 1 : N - 1;
omega2 = -4 * pi * m * ((N - k2) / N) .^ 2;
omega = [omega1 omega2];

% Make spectra
H = exp(0 + i * omega);

% inverse FFT
y = ifft(H);

y = real(y); % remaining imag. part of calc errors
y = circshift(y, [0, -m]); % drift start point

% confirm TSP waveform
hold off;
n = 0 : N - 1;
figure(1);
plot(n(1 : length(n)), y(1 : length(y)));

% write to file
% y = y ./ max(y); % 0dBFS normalize
y = y ./ max(y) .* 0.3; % 0dBFS normalize and scaling (x0.3)
y = y(f:g); % cut
y2 = [y; y]'; % duplicate channels for 2ch wav
% audiowrite('tsp_24bit_96k_2ch.wav', y2, Fs, 'BitsPerSample', bit)
audiowrite(['tsp_',num2str(bit),'bit_',num2str(Fs/1000.0,'%10.2f'),'k_2ch.wav'], y2, Fs, 'BitsPerSample', bit)

% inverse FFT for inverse TSP
iy = ifft(conj(H));

iy = real(iy); % remaining imag. part of calc errors
iy = circshift(iy, [0, m]); % drift start point

% confirm inverse TSP waveform
hold off;
n = 0 : N - 1;
figure(2);
plot(n(1 : length(n)), iy(1 : length(iy)));

% write to file
iy = iy ./ max(iy); % 0dBFS normalize
% iy = iy ./ max(iy) .* 0.3; % 0dBFS normalize and scaling (x0.3)
iy = iy(f:g); % cut

% for verification of accounts
imp = conv(y, iy);

% iy = [iy; iy]'; % duplicate channels for 2ch wav
% audiowrite('itsp_24bit_48k.wav', iy, Fs, 'BitsPerSample', bit)
iy = single(iy);
% audiowrite('itsp_float_48k.wav', iy, Fs, 'BitsPerSample', 32)
audiowrite(['itsp_float_',num2str(Fs/1000.0,'%10.2f'),'k.wav'], iy, Fs, 'BitsPerSample', 32)

% confirm impluse waveform
hold off;
figure(3);
plot(imp);

